

// Motion alarm using IR remote control.
// "Arm" and "Disarm" the alarm by pressing the "X" button on the IR remote.
// Once triggered, press any other button on the remote to turn off the alarm.

/*

Plumduino
Plumduino_BaseSketch_Rev01_05
Version 1.5 08/2016

This is a basic sketch that can be used as a starting point
for various functionality of the Plumduino board.

This code was written by Kevin King for Plum Geek LLC.
Portions from other open source projects where noted.
This code is licensed under:
Creative Commons Attribution-ShareAlike 2.0 Generic (CC BY-SA 2.0)
https://creativecommons.org/licenses/by-sa/2.0/
Visit http://www.plumgeek.com for Plumduino information.
Visit http://www.arduino.cc to learn about the Arduino.

*/

#include "PlumduinoHardware.h"  // Leave this line first. Do not edit this line. This causes Arduino
                                // to include background functions when turning your code into
                                // machine language the Plumduino processor can understand.



// Notice in this example we are declaring the variables BEFORE the setup() function. This is
// good practice because you can call functions and refer to these variables inside the setup()
// function itself. If you don't delcare them before setup() you will get compile errors. This
// is necessary in this case because we're going to play the "disarm" sound at the start of the
// behavior so the lights indicated the alarm is disarmed from the start.


// Edit these values to control some aspects of the behavior.
int blinkInterval = 2000;       //number of milliseconds between "blinks" when armed
int blinkDuration = 20;         //during of the "blink" in milliseconds

// Other variable declarations. No need to edit any of these.
byte button;                    //holder for IR remote button
bool armed = 0;                 //will be "0" or "False" if not armed, "1" or "True" if armed
unsigned long lastTimeStamp=0;  //used as a holder for result of millis(), useful for counting
                                //timing intervals. Read Arduino millis() page for more info
                                //https://www.arduino.cc/en/Reference/Millis

                                
void setup(){
  hardwareBegin();        //initialize Plumduino processor to work with the board circuitry 
  ChirpDisarmedSound();   //indicate alarm is disarmed from the start
  RxIRRestart(4);         //prepare to receive 4 byte IR remote packet
}

                                
void loop() {

  //Check to see if a packet has been received from the IR remote
  if(IsIRDone()){             //if an IR packet has been received....
    button = GetIRButton();   //get button number
    if (button == 21){        //if button was 21 (the "X" button)....
      ToggleArmedState();     //toggle Armed State (you can edit this function down below)
    }
    delay(200);               //short delay to allow time to take finger off the button before resetting button
    RxIRRestart(4);           //prepare to receive another 4 byte IR remote packet
  }

  //Check to see if the "blink" time interval has passed
  if(millis() > lastTimeStamp + blinkInterval){ //useful method of coutning off timing intervals
    lastTimeStamp = millis(); //mark new time stamp
    if(armed){
      BlinkArmedLED();        //blink the "armed" LED only if presently armed (edit this function below)
    }
  }

  //Check to see if we should sound the alrarm
  if(armed && digitalRead(Motion)){ //if alarm is armed *and* Motion sensor line is high....
    SoundTheAlarm();                //go sound the alarm (edit this fucntion down below to customize)
  }
                                 
} //closing curly of the “loop()” function


void BlinkArmedLED(void){   //this function runs after each "blinkInterval" passes
  setPixelRGB(0,30,0,0);    //pixels 0 and 7 will blink indicating it is armed
  setPixelRGB(7,30,0,0);    //change these to different colors, LEDs, or patterns as you like
  delay(blinkDuration);     //duration of time blink will remain on
  offPixels();              //turn all the pixels back off
}

void ToggleArmedState(void){  //toggles between "armed" and "disarmed" states

  if(!armed){       //if *not* already armed, toggle state to "armed" and play chirp 
    armed = 1;              //set armed flag to true
    setAllPixelsRGB(30,0,0);
    while(digitalRead(Motion)){ //if Motion was already triggered for some reason ...
      //... do nothing (wait for Motion trigger line to go back low)
    }
    ChirpArmedSound();      //indicate alarm is now "armed"
  }
  else{             //otherwise, alarm must have already been armed, so we will disarm it instead
    armed = 0;              //set armed flag to false
    ChirpDisarmedSound();   //indicate alrm is now "disarmed"
  }
}

void ChirpArmedSound(void){     //customize this function to set how alarm alerts to being armed
  playChirp(7000);
  setAllPixelsRGB(50,0,0);      //all pixels Red
  delay(50);
  offChirp();
  offPixels();                  //turn off all pixels

  delay(100);                   //delay between light & sound blips
  
  playChirp(7000);
  setAllPixelsRGB(50,0,0);      //all pixels Red
  delay(50);
  offChirp();
  offPixels();                  //turn off all pixels
}

void ChirpDisarmedSound(void){  //customize this function to set how alarm alerts to being disarmed
  playChirp(4700);              //play a low pitched "disarm" sound
  offPixels();                  //make sure we start with all pixels turned off
  setPixelRGB(0,0,25,0);        //set outer pixes to green to indicate "disarmed"
  setPixelRGB(7,0,25,0);
  delay(300);
  playChirp(1500);
  delay(500);
  offChirp();
}

void SoundTheAlarm(void){   //behavior that will run after alarm is activated

  //This function could be written more efficiently. It is spread out so it is easier to
  //read and customize. You could mostly re-write this functiont do anything you want when
  //the alarm is activated. This version plays a tone and turns on the lights, then checks
  //to see if a recognized IR remote key has been pressed to disarm the alarm. This happens
  //twice, with a differnet tone and color set in the middle.
  
  RxIRRestart(4);                           //give the IR receive function a fresh reset to make sure it's running
  lastTimeStamp = millis();                 //mark new time stamp

  while(millis() < lastTimeStamp + 30000){  //this will run until either 30 seconds passes
                                            //or the break; command is run, which will break
                                            //out of the "while" loop. The 30 seconds will
                                            //make sure the alarm only runs for 30 seconds, so
                                            //when your cat sets it off and you're not around,
                                            //it won't annoy your neighbors for the next 3 days.
                                           
    playChirp(4000);
    setAllPixelsRGB(0,0,200);
    delay(500);

    if(IsIRDone()){
      button = GetIRButton();
      if(button){             //if they key was recognized (otherwise "button" will be "0", or "false")
        RxIRRestart(4);
        break;                //a recognized key was seen, break out of "while" loop
      }
      else{
        RxIRRestart(4);
      }
    } // end of if(IsIRDone)
    
    playChirp(6000);
    setAllPixelsRGB(200,0,0);
    delay(500);

    if(IsIRDone()){
      button = GetIRButton(); 
      if(button){             //if they key was recognized (otherwise "button" will be "0", or "false")
        RxIRRestart(4);
        break;                //a recognized key was seen, break out of "while" loop
      }
      else{
        RxIRRestart(4);
      }
    } // end of if(IsIRDone)
  } // end of while(millis())

  ToggleArmedState();         //disarm the alarm before exiting
  offChirp();                 //make sure chirp is off at end of sequence
}  


